@@ -309,10 +309,8 @@ Cardinal * num_params;
  */
 
 static void
-InitializeSearchWidget(search, dir, replace_active)
-struct SearchAndReplace * search;
-XawTextScanDirection dir;
-Boolean replace_active;
+InitializeSearchWidget(struct SearchAndReplace *search,
+  XawTextScanDirection dir, Boolean replace_active)
 {
 replace_active = replace_active; /* PH - shuts compilers up */
 
@@ -503,10 +501,8 @@ struct SearchAndReplace * search;
  */
 
 static void
-SetSearchLabels(search, msg1, msg2, bell)
-struct SearchAndReplace * search;
-String msg1, msg2;
-Boolean bell;
+SetSearchLabels(struct SearchAndReplace *search, String msg1, String msg2,
+  Boolean bell)
 {
 msg2 = msg2; /* PH - shuts compilers up */
   (void) SetResource( search->label1, XtNlabel, (XtArgVal) msg1);
