@@ -2,7 +2,7 @@
 *                  Exim Monitor                  *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2001 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -474,7 +474,7 @@ c = c;
 
 XtPopdown((Widget)dialog_shell);
 XtDestroyWidget((Widget)dialog_shell);
-while (isspace(*s)) s++;
+while (isspace((uschar)*s)) s++;
 if (s[0] != 0)
   {
   if (actioned_message[0] != 0)
