@@ -29,8 +29,9 @@ st='	 '
 
 (sed -n \
   "/\\\$/d;s/#.*\$//;s/^[$st]*\\([A-Z][^:$st]*\\)[$st]*=[$st]*\\([^$st]*\\)[$st]*\$/\\1=\\2 export \\1/p" \
-  < makefile ; echo "./buildconfig") | /bin/sh || exit 1
+  < Makefile ; echo "./buildconfig") | /bin/sh || exit 1
 
 echo ">>> config.h built"
+echo ""
 
 # End of Configure-config.h
