@@ -665,6 +665,7 @@ BOOL last_was_cr = FALSE;
 
 flock_t lock_data;
 error_block *bad_addresses = NULL;
+char *errmsg;
 
 /* Final message to give to SMTP caller */
 
@@ -862,8 +863,8 @@ for (;;)
   dots aren't exactly common. They are legal in RFC 822, though. If the
   following is CRLF or LF, this is the line that that terminates the
   entire message. We set message_ended to indicate this has happened (to
-  prevent further reading), and break out of the loop, leaving a blank line,
-  which will serve to terminate the header. */
+  prevent further reading), and break out of the loop, having got rid
+  of the unwanted header item and set next = NULL. */
 
     {
     if (ptr == 0 && ch == '.' && (smtp_input || dot_ends))
@@ -881,6 +882,8 @@ for (;;)
       if (ch == '\n')
         {
         message_ended = END_DOT;
+        store_reset(next);
+        next = NULL;
         break;
         }
       }
@@ -951,8 +954,7 @@ for (;;)
   message_size++;
 
   /* A blank line signals the end of the headers; release the unwanted
-  space and set next to NULL to indicate this. A terminating line containing
-  just '.' will have been recognized above, and turned into a blank line. */
+  space and set next to NULL to indicate this. */
 
   if (ptr == 1)
     {
@@ -1335,8 +1337,7 @@ if (extract_recip)
       while (*s != 0)
         {
         char *ss = parse_find_address_end(s, FALSE);
-        char *recipient, *errmess;
-        int terminator = *ss;
+        char *recipient, *errmess, *p, *pp;
         int start, end, domain;
 
         /* Check on maximum */
@@ -1353,11 +1354,16 @@ if (extract_recip)
               FALSE)? error_rc : EXIT_FAILURE);
           }
 
-        /* Temporarily terminate the string at this point, and extract
-        the operative address within. */
+        /* Make a copy of the address, and remove any internal newlines. These
+        may be present as a result of continuations of the header line. The
+        white space that follows the newline must not be removed - it is part
+        of the header. */
 
-        *ss = 0;
-        recipient = parse_extract_address(s,&errmess,&start,&end,&domain,FALSE);
+        pp = recipient = store_get(ss - s + 1);
+        for (p = s; p < ss; p++) if (*p != '\n') *pp++ = *p;
+        *pp = 0;
+        recipient = parse_extract_address(recipient, &errmess, &start, &end,
+          &domain, FALSE);
 
         /* Keep a list of all the bad addresses so we can send a single
         error message at the end. However, an empty address is not an error;
@@ -1393,10 +1399,9 @@ if (extract_recip)
             extracted_ignored = TRUE;
           }
 
-        /* Put back the terminator */
+        /* Move on past this address */
 
-        *ss = terminator;
-        s = ss + (terminator? 1:0);
+        s = ss + (*ss? 1:0);
         while (isspace((uschar)*s)) s++;
         }
 
@@ -1589,49 +1594,14 @@ header_list->text = string_sprintf("%s; %s\n", received, timestamp);
 header_list->slen = (int)strlen(header_list->text);
 header_list->type = htype_received;
 
-/* If there are any rewriting rules, apply them to the sender address, unless
-it has already been rewitten as part of verification for SMTP input. */
 
-if (global_rewrite_rules != NULL && !sender_address_rewritten &&
-    sender_address[0] != 0)
-  {
-  sender_address = rewrite_address(sender_address, FALSE, TRUE,
-    global_rewrite_rules, rewrite_existflags);
-  DEBUG(5) debug_printf("rewritten sender = %s\n", sender_address);
-  }
-
-
-/* The headers must all be run through rewrite_header, because it ensures that
-addresses are fully qualified, as well as applying any rewriting rules that may
-exist. Take care to keep the from_header pointer correct if it gets re-written.
-Note: start at second header; no point wasting time on Received. This rewriting
-is documented as happening *after* recipient addresses are taken from the
-headers by the -t command line option. */
-
-for (h = header_list->next; h != NULL; h = h->next)
-  {
-  header_line *newh = rewrite_header(h, NULL, NULL, global_rewrite_rules,
-    rewrite_existflags, TRUE);
-  if (newh != NULL)
-    {
-    if (h == from_header) from_header = newh;
-    h = newh;
-    }
-  }
-
-search_tidyup();    /* Free any cached resources */
-
-
-/* At this point, the only one of the saved header variables containing
-addresses that is still valid if not NULL is from_header. We haven't bother to
-updated any of the others, since the code below merely tests for their being
-NULL. Beware, though, if any changes require anything else. */
-
-
-/* If there is no from: header, generate one. If there is no sender address,
+/* If there is no From: header, generate one. If there is no sender address,
 but the sender is local or this is a local delivery error, use the originator
 login. This shouldn't happen for genuine bounces, but might happen for
-autoreplies. */
+autoreplies. The addition of From: must be done *before* checking for the
+possible addition of a Sender: header, because untrusted_set_sender allows an
+untrusted user to set anything in the envelope (which might then get info
+From:) but we still want to ensure a valid Sender: if it is required. */
 
 if (from_header == NULL)
   {
@@ -1647,11 +1617,12 @@ if (from_header == NULL)
         originator_login,
         qualify_domain_sender,
         (originator_name[0] == 0)? "" : ">");
-      from_header = header_last;
       }
     }
 
-  /* There is a non-null envelope sender */
+  /* There is a non-null envelope sender. Build the header using the original
+  sender address, before any rewriting that might have been done while
+  verifying it. */
 
   else
     {
@@ -1660,25 +1631,27 @@ if (from_header == NULL)
         header_names[hn_from].name,
         originator_name,
         (originator_name[0] == 0)? "" : " <",
-        sender_address,
+        (sender_address_unrewritten == NULL)?
+          sender_address : sender_address_unrewritten,
         (originator_name[0] == 0)? "" : ">");
     else
       header_add(htype_from, "%s: %s\n", header_names[hn_from].name,
         sender_address);
-    from_header = header_last;
+    from_header = header_last;    /* To get it checked for Sender: */
     }
   }
 
-/* If the sender is local, check that From: is correct, and if not, generate a
-Sender: header, unless disabled. Any previously-existing Sender: header was
-removed above. Note that sender_local, as well as being TRUE if the caller of
-exim is not trusted, is also true if a trusted caller did not supply a -f
-argument for non-smtp input. To allow trusted callers to forge From: without
-supplying -f, we have to test explicitly here. If the From: header contains
-more than one address, then the call to parse_extract_address fails, and a
-Sender: header is inserted, as required. */
 
-if (sender_local && !trusted_caller && local_from_check)
+/* If the sender is local, check that an existing From: is correct, and if not,
+generate a Sender: header, unless disabled. Any previously-existing Sender:
+header was removed above. Note that sender_local, as well as being TRUE if the
+caller of exim is not trusted, is also true if a trusted caller did not supply
+a -f argument for non-smtp input. To allow trusted callers to forge From:
+without supplying -f, we have to test explicitly here. If the From: header
+contains more than one address, then the call to parse_extract_address fails,
+and a Sender: header is inserted, as required. */
+
+if (sender_local && local_from_check && !trusted_caller && from_header != NULL)
   {
   BOOL make_sender = TRUE;
   int start, end, domain;
@@ -1712,18 +1685,44 @@ if (sender_local && !trusted_caller && local_from_check)
         make_sender = FALSE;
     }
 
-  /* We have to cause the Sender header to be rewritten if there are
-  appropriate rewriting rules. */
-
   if (make_sender)
-    {
     header_add(htype_sender, "%s: %s <%s>\n",
       header_names[hn_sender].name, originator_name, login_sender_address);
-    (void) rewrite_header(header_last, NULL, NULL, global_rewrite_rules,
-      rewrite_existflags, TRUE);
-    }
   }
 
+
+/* If there are any rewriting rules, apply them to the sender address, unless
+it has already been rewritten as part of verification for SMTP input. */
+
+if (global_rewrite_rules != NULL && sender_address_unrewritten == NULL &&
+    sender_address[0] != 0)
+  {
+  sender_address = rewrite_address(sender_address, FALSE, TRUE,
+    global_rewrite_rules, rewrite_existflags);
+  DEBUG(5) debug_printf("rewritten sender = %s\n", sender_address);
+  }
+
+
+/* The headers must always be run through rewrite_header, because it ensures
+that addresses are fully qualified, as well as applying any rewriting rules
+that may exist. Note: start at the second header; no point wasting time on
+Received. This rewriting is documented as happening *after* recipient addresses
+are taken from the headers by the -t command line option. An added Sender: gets
+rewritten here. */
+
+for (h = header_list->next; h != NULL; h = h->next)
+  {
+  header_line *newh = rewrite_header(h, NULL, NULL, global_rewrite_rules,
+    rewrite_existflags, TRUE);
+  if (newh != NULL) h = newh;
+  }
+
+
+/* WARNING: At this point, the saved header variables contain addresses that
+are no longer valid, because we haven't bothered to update them on rewriting.
+This is because the code below merely tests for their being NULL. */
+
+
 /* A message is not legal unless it has at least one of "to", "cc", or "bcc".
 Note that although the minimal examples in RFC822 show just "to" or "bcc", the
 full syntax spec allows "cc" as well. If any resent- header exists, this
@@ -1806,6 +1805,8 @@ DEBUG(9)
   if (next != NULL) debug_printf("%s", next->text);
   }
 
+search_tidyup();    /* Free any cached resources */
+
 /* The headers are now complete in store. If we are running in filter
 testing mode, that is all this function does. Return TRUE if the message
 ended with a dot. */
@@ -2029,6 +2030,7 @@ format); write it (remembering that it might contain binary zeros). The result
 of fwrite() isn't inspected; instead we call ferror() below. */
 
 fprintf(data_file, "%s-D\n", message_id);
+
 if (next != NULL)
   {
   char *s = next->text;
@@ -2282,14 +2284,9 @@ this message. Ensure the first time flag is set in the newly-received message.
 
 deliver_firsttime = TRUE;
 
-if ((msg_size = spool_write_header(message_id, FALSE)) < 0)
+if ((msg_size = spool_write_header(message_id, SW_RECEIVING, &errmsg)) < 0)
   {
-  char *msg = string_sprintf("Spool write error (%s) while receiving message "
-    "from %s",
-    strerror(errno),
-    (sender_fullhost != NULL)? sender_fullhost : sender_ident);
-
-  log_write(0, LOG_MAIN, "Message abandoned: %s", msg);
+  log_write(0, LOG_MAIN, "Message abandoned: %s", errmsg);
 
   unlink(spool_name);           /* Lose the data file */
 
@@ -2305,11 +2302,11 @@ if ((msg_size = spool_write_header(message_id, FALSE)) < 0)
       {
       error_block eblock;
       eblock.next = NULL;
-      eblock.text1 = msg;
+      eblock.text1 = errmsg;
       if (!moan_to_sender(ERRMESS_IOERR, &eblock, header_list, data_file,
         FALSE)) error_rc = EXIT_FAILURE;
       }
-    else fprintf(stderr, "exim: %s\n", msg);
+    else fprintf(stderr, "exim: %s\n", errmsg);
     fclose(data_file);
     exim_exit(error_rc);
     }
@@ -2503,7 +2500,7 @@ if (smtp_input)
   nothing on success. The function moan_smtp_batch() does not return -
   it exits from the program with a non-zero return code. */
 
-  else if (smtp_reply != NULL) moan_smtp_batch(NULL, smtp_reply);
+  else if (smtp_reply != NULL) moan_smtp_batch(NULL, "%s", smtp_reply);
   }
 
 /* Reset headers so that logging of rejects for a subsequent message doesn't
