@@ -257,10 +257,13 @@ if (yield != OK) return yield;
 
 if (!direct_get_ugid(dblock, "director", addr, &ugid)) return ERROR;
 
-/* Set up the errors address for the children of this address. */
+/* Set up the errors address for the children of this address. Preserve it in
+the parent address - this is used if a transport exists, and also after
+"unseen" handling of any of the children. */
 
 rc = direct_get_errors_address(addr, dblock, "director", verify, &errors_to);
 if (rc != OK) return rc;
+addr->errors_address = errors_to;
 
 /* Set up the additional and removeable headers for the children of this
 address. */
@@ -281,7 +284,7 @@ address to use that transport. Ignore the alias text. */
 if (transport != NULL)
   {
   addr->transport = transport;
-  addr->errors_address = errors_to;
+/*  addr->errors_address = errors_to;  */
   addr->home_dir = dblock->home_directory;
   addr->current_dir = dblock->current_directory;
   addr->extra_headers = extra_headers;
@@ -365,7 +368,7 @@ df_generated(dblock, addr_new, addr, generated, errors_to, extra_headers,
 it means that there was just a :blackhole: entry in the alias file. Log
 something. */
 
-if (addr->child_count <= 0 && !verify && !address_test_mode)
+if (deliver_firsttime && addr->child_count <= 0 && !verify && !address_test_mode)
   log_write(0, LOG_MAIN, "=> :blackhole: <%s> D=%s", addr->orig, dblock->name);
 
 /* Put the original address onto the succeed queue. This ensures that any
