@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2000 */
+/* Copyright (c) University of Cambridge 1995 - 2001 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
@@ -45,8 +45,8 @@ switch (extracted)
   return DEFER;
 
   case EXTRACTED_UNKNOWN:
-  DEBUG(2) debug_printf("%s director failed by :unknown:\n", dblock->name);
-  return FAIL;
+  DEBUG(2) debug_printf("%s director declined by :unknown:\n", dblock->name);
+  return DECLINE;
 
   case EXTRACTED_FAIL:
   addr->message = string_sprintf("forced failure: %s", error);
