@@ -66,7 +66,7 @@ while (generated != NULL)
   next->next = *addr_new;
   *addr_new = next;
 
-  if (ob->one_time)
+  if (ob->one_time && !queue_2stage)
     {
     for (parent = addr; parent->parent != NULL; parent = parent->parent);
     next->onetime_parent = parent->orig;
