@@ -911,12 +911,15 @@ if (yield != OK) return yield;
 
 /* Handle the generated addresses; set up the errors address and the additional
 and removeable headers, and then add the addresses to the addr_new chain, using
-the common function. */
+the common function. We save the errors_address in the original parent, in case
+any of the children are routed/directed in an "unseen" manner. */
 
 if (generated != NULL)
   {
   rc = direct_get_errors_address(addr, dblock, "director", verify, &errors_to);
   if (rc != OK) return rc;
+  addr->errors_address = errors_to;
+
 
   rc = direct_get_munge_headers(addr, dblock, "director", &extra_headers,
     &remove_headers);
