@@ -363,4 +363,18 @@ been defined, set them to the null string. */
   #define PID_FILE_PATH ""
 #endif
 
+/* The EDQUOT error code isn't universally available, though it is widespread.
+There is a particular shambles in SunOS5, where it did not exist originally,
+but got installed with a particular patch for Solaris 2.4. There is a
+configuration variable for specifying what the system's "over quota" error is,
+but if it is not set, default to EDQUOT if it exists, otherwise ENOSPC. */
+
+#ifndef ERRNO_QUOTA
+#ifdef  EDQUOT
+#define ERRNO_QUOTA EDQUOT
+#else
+#define ERRNO_QUOTA ENOSPC
+#endif
+#endif
+
 /* End of exim.h */
