@@ -15,6 +15,7 @@ Argument: the name of the lock file
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <signal.h>
 #include <errno.h>
 #include <time.h>
 #include <netdb.h>
@@ -29,23 +30,49 @@ Argument: the name of the lock file
 #define TRUE  1
 
 
+/* Flag for timeout signal handler */
+
+static int sigalrm_seen = FALSE;
+
+
+/* We need to pull in strerror() and os_non_restarting_signal() from the
+os.c source, if they are required for this OS. However, we don't need any of
+the other stuff in os.c, so force the other macros to omit it. */
+
+#ifndef OS_RESTARTING_SIGNAL
+  #define OS_RESTARTING_SIGNAL
+#endif
+
+#ifndef OS_STRSIGNAL
+  #define OS_STRSIGNAL
+#endif
+
+#ifndef OS_STREXIT
+  #define OS_STREXIT
+#endif
+
+#ifndef OS_LOAD_AVERAGE
+  #define OS_LOAD_AVERAGE
+#endif
+
+#ifndef FIND_RUNNING_INTERFACES
+  #define FIND_RUNNING_INTERFACES
+#endif
+
+#include "../src/os.c"
+
+
 
-#ifdef STRERROR_FROM_ERRLIST
 /*************************************************
-*     Provide strerror() for non-ANSI libraries  *
+*             Timeout handler                    *
 *************************************************/
 
-/* Some old-fashioned systems still around (e.g. SunOS4) don't have strerror()
-in their libraries, but can provide the same facility by this simple
-alternative function. */
-
-char *
-strerror(int n)
+static void
+sigalrm_handler(int sig)
 {
-if (n < 0 || n >= sys_nerr) return "unknown error number";
-return sys_errlist[n];
+sig = sig;      /* Keep picky compilers happy */
+sigalrm_seen = TRUE;
 }
-#endif /* STRERROR_FROM_ERRLIST */
 
 
 
@@ -53,14 +80,48 @@ return sys_errlist[n];
 *           Give usage and die                   *
 *************************************************/
 
-void usage(void)
+static void
+usage(void)
 {
-printf("usage: exim_lock [-v] [-q] [-lockfile] [-fcntl | -mbx] <file name> [command]\n");
+printf("usage: exim_lock [-v] [-q] [-lockfile] [-fcntl | -mbx]\n"
+       "       [-retries <n>] [-interval <n>] [-timeout <n>]\n"
+       "       <file name> [command]\n");
 exit(1);
 }
 
 
 
+/*************************************************
+*         Apply a lock to a file descriptor      *
+*************************************************/
+
+static int
+apply_lock(int fd, int type, int timeout)
+{
+int yield;
+struct flock lock_data;
+lock_data.l_type = type;
+lock_data.l_whence = lock_data.l_start = lock_data.l_len = 0;
+
+sigalrm_seen = FALSE;
+
+if (timeout > 0)
+  {
+  int save_errno;
+  os_non_restarting_signal(SIGALRM, sigalrm_handler);
+  alarm(timeout);
+  yield = fcntl(fd, F_SETLKW, &lock_data);
+  save_errno = errno;
+  alarm(0);
+  signal(SIGALRM, SIG_IGN);
+  errno = save_errno;
+  }
+else yield = fcntl(fd, F_SETLK, &lock_data);
+
+return yield;
+}
+
+
 
 /*************************************************
 *           The exim_lock program                *
@@ -69,13 +130,14 @@ exit(1);
 int main(int argc, char **argv)
 {
 int lock_retries = 10;
-int lock_interval = 5;
+int lock_interval = 3;
+int lock_fcntl_timeout = 0;
 int use_lockfile = FALSE;
 int use_fcntl = FALSE;
 int use_mbx = FALSE;
 int verbose = FALSE;
 int quiet = FALSE;
-int i, j, len, rc;
+int i, j, len;
 int fd = -1;
 int hd = -1;
 int md = -1;
@@ -85,7 +147,6 @@ char *filename;
 char *lockname = NULL, *hitchname = NULL;
 char *primary_hostname, *command;
 struct utsname s;
-struct flock lock_data;
 char buffer[256];
 char tempname[256];
 
@@ -100,8 +161,17 @@ for (i = 1; i < argc; i++)
   else if (strcmp(arg, "-mbx") == 0) use_mbx = TRUE;
   else if (strcmp(arg, "-v") == 0) verbose = TRUE;
   else if (strcmp(arg, "-q") == 0) quiet = TRUE;
+  else if (++i < argc)
+    {
+    int value = atoi(argv[i]);
+    if (strcmp(arg, "-retries") == 0) lock_retries = value;
+    else if (strcmp(arg, "-interval") == 0) lock_interval = value;
+    else if (strcmp(arg, "-timeout") == 0) lock_fcntl_timeout = value;
+    else usage();
+    }
   else usage();
   }
+
 if (quiet) verbose = 0;
 
 /* Default is to use both non-MBX kinds of lock; fcntl and mbx are exclusive */
@@ -157,7 +227,7 @@ if (use_lockfile)
   {
   if (uname(&s) < 0)
     {
-    printf("exim_lock: failed to find host name using uname()");
+    printf("exim_lock: failed to find host name using uname()\n");
     exit(1);
     }
   primary_hostname = s.nodename;
@@ -178,6 +248,7 @@ if (use_lockfile)
 
 for (j = 0; j < lock_retries; j++)
   {
+  int sleep_before_retry = TRUE;
   struct stat statbuf;
 
   /* Try to build a lock file if so configured */
@@ -210,6 +281,8 @@ for (j = 0; j < lock_retries; j++)
     if (!quiet) printf("exim_lock: lock file successfully created\n");
     }
 
+  /* We are done if no other locking required. */
+
   if (!use_fcntl && !use_mbx) break;
 
   /* Open the file for writing. */
@@ -223,16 +296,18 @@ for (j = 0; j < lock_retries; j++)
     goto CLEAN_UP;
     }
 
-  /* Lock using fcntl. This is done with a non-blocking call because that's the
-  way Exim does it. It is safer not to keep the file open while waiting for the
-  lock - especially on NFS file systems. */
+  /* If there is a timeout, implying blocked locking, we don't want to
+  sleep before any retries after this. */
+
+  if (lock_fcntl_timeout > 0) sleep_before_retry = FALSE;
+
+  /* Lock using fcntl. There are pros and cons to using a blocking call vs
+  a non-blocking call and retries. Exim is non-blocking by default, but setting
+  a timeout changes it to blocking. */
 
   if (!use_mbx)
     {
-    lock_data.l_type = F_WRLCK;
-    lock_data.l_whence = lock_data.l_start = lock_data.l_len = 0;
-
-    if (fcntl(fd, F_SETLK, &lock_data) >= 0)
+    if (apply_lock(fd, F_WRLCK, lock_fcntl_timeout) >= 0)
       {
       if (!quiet) printf("exim_lock: fcntl() lock successfully applied\n");
       break;
@@ -251,23 +326,20 @@ for (j = 0; j < lock_retries; j++)
 
   else
     {
-    lock_data.l_type = F_RDLCK;
-    lock_data.l_whence = lock_data.l_start = lock_data.l_len = 0;
-    rc = fcntl(fd, F_SETLK, &lock_data);
-
-    if (rc >= 0)
+    if (apply_lock(fd, F_RDLCK, lock_fcntl_timeout) >= 0)
       {
       if (!quiet) printf("exim_lock: fcntl() read lock successfully applied\n");
       }
     else
       {
-      printf("exim_lock: fcntl() read lock failed: %s", strerror(errno));
+      printf("exim_lock: fcntl() read lock failed: %s\n", strerror(errno));
       goto RETRY;
       }
 
     if (fstat(fd, &statbuf) < 0)
       {
-      printf("exim_lock: fstat() of %s failed: %s", filename, strerror(errno));
+      printf("exim_lock: fstat() of %s failed: %s\n", filename,
+        strerror(errno));
       yield = 1;
       goto CLEAN_UP;
       }
@@ -303,9 +375,7 @@ for (j = 0; j < lock_retries; j++)
 
     chmod (tempname, 0600);
 
-    lock_data.l_type = F_WRLCK;
-    lock_data.l_whence = lock_data.l_start = lock_data.l_len = 0;
-    if (fcntl(md, F_SETLK, &lock_data) >= 0)
+    if (apply_lock(md, F_WRLCK, lock_fcntl_timeout) >= 0)
       {
       if (!quiet) printf("exim_lock: fcntl() lock successfully applied to mbx "
         "lock file %s\n", tempname);
@@ -313,7 +383,8 @@ for (j = 0; j < lock_retries; j++)
       }
     else
       {
-      printf("exim_lock: fcntl() failed for %s: %s", tempname, strerror(errno));
+      printf("exim_lock: fcntl() failed for %s: %s\n", tempname,
+        strerror(errno));
       goto RETRY;
       }
     }
@@ -349,10 +420,21 @@ for (j = 0; j < lock_retries; j++)
     hd = -1;
     }
 
-  /* Wait a bit before retrying. */
+  /* If a blocking call timed out, break the retry loop if the total time
+  so far is not less than than retries * interval. */
 
-  printf(" ... waiting\n");
-  sleep(lock_interval);
+  if (sigalrm_seen &&
+      (j + 1) * lock_fcntl_timeout >= lock_retries * lock_interval)
+    j = lock_retries;
+
+  /* Wait a bit before retrying, except when it was a blocked fcntl() that
+  caused the problem. */
+
+  if (j < lock_retries && sleep_before_retry)
+    {
+    printf(" ... waiting\n");
+    sleep(lock_interval);
+    }
   }
 
 if (j >= lock_retries)
@@ -384,12 +466,14 @@ else
 system(command);
 
 /* Remove the locks and exit. Unlink the /tmp file if we can get an exclusive
-lock on the mailbox. */
+lock on the mailbox. This should be a non-blocking lock call, as there is no
+point in waiting. */
 
 CLEAN_UP:
 
 if (md >= 0)
   {
+  struct flock lock_data;
   lock_data.l_type = F_WRLCK;
   lock_data.l_whence = lock_data.l_start = lock_data.l_len = 0;
   if (fcntl(fd, F_SETLK, &lock_data) >= 0)
