@@ -2566,7 +2566,7 @@ Returns:      TRUE if no errors
 BOOL
 filter_runtest(int fd, BOOL is_system, BOOL dot_ended)
 {
-int rc, body_len, action;
+int rc, body_len, action, header_size;
 register int ch;
 BOOL yield, delivered;
 struct stat statbuf;
@@ -2658,6 +2658,7 @@ message_body_end = store_malloc(message_body_visible + 1);
 s = message_body_end;
 body_len = 0;
 body_linecount = 0;
+header_size = message_size;
 
 if (!dot_ended && !feof(stdin))
   {
@@ -2713,6 +2714,7 @@ if (!dot_ended && !feof(stdin))
   }
 
 message_body[body_len] = 0;
+message_body_size = message_size - header_size;
 
 /* body_len stops at message_body_visible; it if got there, we may have
 wrapped round in message_body_end. */
