@@ -335,6 +335,8 @@ header_line *header_last      = NULL;
 header_line *header_list      = NULL;
 header_name *header_names     = NULL;
 
+/* These are the headers that Exim notices, and may use in various ways. */
+
 header_name header_names_normal[] = {
   { "Bcc",         3 },       /* This data must be in the correct */
   { "Cc",          2 },       /* order as defined by the enum for */
@@ -347,6 +349,8 @@ header_name header_names_normal[] = {
   { "Subject",     7 }
 };
 
+/* If any resent- header exists, this set of names is used instead. */
+
 header_name header_names_resent[] = {
   { "Resent-Bcc",        10 }, /* This data must be in the correct */
   { "Resent-Cc",          9 }, /* order as defined by the enum for */
@@ -355,8 +359,18 @@ header_name header_names_resent[] = {
   { "Resent-Message-Id", 17 },
   { "Resent-Sender",     13 },
   { "Resent-To",          9 },
+  { "Reply-To",           8 }, /* There aren't resent-versions of these */
+  { "Subject",            7 }
+
+/*****
+Exim used to have these two lines: they are replaced by the above because
+RFC 2822 has disallowed Resent-Reply-To: and there never was a Resent-Subject:
+(I was deluded).
+
   { "Resent-Reply-To",   15 },
   { "Resent-Subject",    14 }
+*****/
+
 };
 
 BOOL   headers_check_syntax   = FALSE;
@@ -420,6 +434,7 @@ BOOL   log_received_sender    = FALSE;
 BOOL   log_received_recipients = FALSE;
 BOOL   log_refused_recipients = FALSE;
 BOOL   log_rewrites           = FALSE;
+BOOL   log_sender_on_delivery = FALSE;
 BOOL   log_smtp_confirmation  = FALSE;
 BOOL   log_smtp_connections   = FALSE;
 BOOL   log_smtp_syntax_errors = FALSE;
@@ -645,7 +660,7 @@ char  *sender_address         = NULL;
 BOOL   sender_address_forced  = FALSE;
 char  *sender_address_relay   = NULL;
 char  *sender_address_relay_hosts = "*";
-BOOL   sender_address_rewritten = FALSE;
+char  *sender_address_unrewritten = NULL;
 char  *sender_fullhost        = NULL;
 char  *sender_helo_name       = NULL;
 char **sender_host_aliases    = NULL;
@@ -713,6 +728,8 @@ char  *stderr_filename        = STDERR_FILE
 int    store_pool             = POOL_MAIN;
 BOOL   strip_excess_angle_brackets = FALSE;
 BOOL   strip_trailing_dot     = FALSE;
+BOOL   syslog_timestamp       = TRUE;
+
 
 int    timeout_frozen_after   = 0;
 BOOL   timestamps_utc         = FALSE;
