@@ -159,7 +159,12 @@ ident_t *id_open __P3(struct sockaddr *, lsock,
 	FD_SET(id->fd, &ws);
 	FD_SET(id->fd, &es);
 
-#if defined(__hpux) || defined(__H3050) || defined(__H3050R)
+/* Modern versions of HP-UX use the correct types for the arguments
+of select(). Older versions do not. Luckily older versions also define
+"hpux" whereas modern versions use only "__hpux" (to be more standard).
+Changed this test from __hpux to hpux on 2-Feb-99 (PH). */
+
+#if defined(hpux) || defined(__H3050) || defined(__H3050R)
 	if ((res = select(FD_SETSIZE, (int *) &rs, (int *) &ws, (int *) &es, timeout)) < 0)
 #else
 	if ((res = select(FD_SETSIZE, &rs, &ws, &es, timeout)) < 0)
