@@ -180,7 +180,8 @@ if ((rc = ldap_url_parse(ldap_url, &ludp)) != 0)
 
 /* If the host name is empty, take it from the separate argument, if
 one is given. OpenLDAP 2.0.6 sets an unset hostname to "" rather than
-empty, but expects NULL later to mean "default", annoyingly. */
+empty, but expects NULL later to mean "default", annoyingly. OpenLDAP 2.0.11
+seems to have fixed this. */
 
 if ((ludp->lud_host == NULL || ludp->lud_host[0] == 0) && server != NULL)
   {
@@ -190,7 +191,7 @@ if ((ludp->lud_host == NULL || ludp->lud_host[0] == 0) && server != NULL)
 else
   {
   host = ludp->lud_host;
-  if (host[0] == 0) host = NULL;
+  if (host != NULL && host[0] == 0) host = NULL;
   port = ludp->lud_port;
   }
 
