@@ -876,7 +876,7 @@ to this function (caseless == TRUE), but if it encounters a magic item
 "+caseful" in the list, it restores a caseful copy from the original address.
 */
 
-strcpy(address, origaddress);
+sprintf(address, "%.*s", big_buffer_size-1, origaddress);
 for (p = address + ((caseless || llen < 0)? 0 : llen); *p != 0; p++)
   *p = tolower(*p);
 
