@@ -726,9 +726,22 @@ if (*s != 0)
     goto PARSE_FAILED;
     }
   }
+
 *start = startptr - (uschar *)mailbox;      /* Return offsets */
 while (isspace(endptr[-1])) endptr--;
 *end = endptr - (uschar *)mailbox;
+
+/* Although this code has no limitation on the length of address extracted,
+other parts of Exim may have limits, and in any case, RFC 2821 limits local
+parts to 64 and domains to 255, so we do a check here, giving an error if the
+address is ridiculously long. */
+
+if (*end - *start > ADDRESS_MAXLENGTH)
+  {
+  *errorptr = string_sprintf("address is ridiculously long: %.64s...", yield);
+  return NULL;
+  }
+
 return (char *)yield;
 
 /* Use goto (via the macro FAILED) to get to here from a variety of places.
