@@ -347,13 +347,14 @@ for (yield = retries; yield != NULL; yield = yield->next)
   otherwise it's a remote host name. Either way, we can persuade the
   match_address function to handle it. */
 
-  if (match_address_list(destination, domain, &dlist, -1, &sep, FALSE)) break;
+  if (match_address_list(destination, domain, TRUE, &dlist, -1, &sep, FALSE))
+    break;
 
   /* If the match failed, have a go at matching on the alternative destination,
   which is normally the domain as opposed to the host for a remote delivery. */
 
   if (alternate != NULL &&
-      match_address_list(alternate, 0, &dlist, -1, &sep, FALSE)) break;
+      match_address_list(alternate, 0, TRUE, &dlist, -1, &sep, FALSE)) break;
   }
 
 *colon = replace;
