@@ -465,21 +465,26 @@ route_unseen(char *name, address_item *addr, address_item **addr_local,
 address_item *parent = deliver_make_addr(addr->orig, TRUE);
 address_item *new = deliver_make_addr(addr->orig, TRUE);
 
+
 /* The invented parent is a copy that replaces the original; note that
-this copies its parent pointer. It has two children. */
+this copies its parent pointer. It has two children, and its errors_address is
+from the original address' parent, if present, otherwise unset. */
 
 *parent = *addr;
 parent->child_count = 2;
+parent->errors_address =
+  (addr->parent == NULL)? NULL : addr->parent->errors_address;
 
 /* The routed/directed address gets a new parent. */
 
 addr->parent = parent;
 
-/* Set up the cloned address and put it on the new address queue */
+/* Set up the cloned address and put it on the new address queue, setting its
+errors address to ignore any errors_to setting on the unseen driver. */
 
 new->parent = parent;
+new->errors_address = parent->errors_address;
 copyflag(new, addr, af_ignore_error|af_forced_local);
-new->errors_address = addr->errors_address;
 new->next = *addr_new;
 *addr_new = new;
 
