@@ -190,7 +190,7 @@ for (;;)
 
   else
     {
-    BOOL removed;
+    BOOL removed = FALSE;
     rc = host_find_bydns(&h, ignore_target_hosts, FALSE, FALSE,
       ob->qualify_single, ob->search_parents, &fully_qualified_name,
       &removed);
