@@ -450,7 +450,11 @@ for (n = 0; n < recipients_count; n++)
   /* Look back from the end for digits/commas and a space, indicating the
   presence of flags, pno data, and possibly the DSN orcpt field.
   The partial attempt at doing DSN was abandoned. This obsolete code is
-  left here just in case. Nothing is documented. */
+  left here just in case. Nothing is documented.
+
+  Jan, 2002: Added code to make it read the Exim 4 version of this, which
+  is just a single number.
+  */
 
   p = big_buffer + nn - 2;
   while (isdigit((uschar)*p) || *p == ',') p--;
@@ -458,7 +462,12 @@ for (n = 0; n < recipients_count; n++)
     {
     int orc;
     *p++ = 0;
-    sscanf(p, "%d,%d,%d", &flags, &pno, &orc);
+    if (sscanf(p, "%d,%d,%d", &flags, &pno, &orc) == 1)
+      {
+      pno = flags;                 /* Convert Exim 4 one-number */
+      flags = rf_onetime;          /* to Exim 3 setting plus a  */
+      orc = 0;                     /* bit in the flags */
+      }
     if (orc > 0)
       {
       p -= orc + 1;
