@@ -509,8 +509,8 @@ if ((options & topt_no_headers) == 0)
 
   if ((options & topt_add_return_path) != 0)
     {
-    char buffer[SENDER_ADDRESS_MAXLENGTH + 20];
-    sprintf(buffer, "Return-path: <%.*s>\n", SENDER_ADDRESS_MAXLENGTH,
+    char buffer[ADDRESS_MAXLENGTH + 20];
+    sprintf(buffer, "Return-path: <%.*s>\n", ADDRESS_MAXLENGTH,
       return_path);
     len = (int)strlen(buffer);
     if (!write_chunk(fd, buffer, len, use_crlf)) return FALSE;
