@@ -236,6 +236,9 @@ if (addr->reply != NULL)
   oncelog = addr->reply->oncelog;
   once_repeat = addr->reply->once_repeat;
   file_expand = addr->reply->file_expand;
+  expand_forbid_exists = addr->reply->forbid_exists;
+  expand_forbid_lookup = addr->reply->forbid_lookup;
+  expand_forbid_perl = addr->reply->forbid_perl;
   return_message = addr->reply->return_message;
   }
 else
@@ -494,6 +497,12 @@ if (ff != NULL)
     if (file_expand)
       {
       char *s = expand_string(big_buffer);
+      DEBUG(9)
+        {
+        if (s == NULL)
+          debug_printf("error while expanding line from file:\n  %s\n  %s\n",
+            big_buffer, expand_string_message);
+        }
       fprintf(f, "%s", (s == NULL)? big_buffer : s);
       }
     else fprintf(f, "%s", big_buffer);
