@@ -487,7 +487,7 @@ addr->basic_errno = 0;
 pointed to by addr->local_part; it starts with the pipe symbol. In other cases,
 the command is supplied as one of the pipe transport's options. */
 
-if (addr->local_part[0] == '|')
+if (testflag(addr,af_pfr) && addr->local_part[0] == '|')
   {
   cmd = addr->local_part + 1;
   while (isspace((uschar)*cmd)) cmd++;
